########## R script: gplsim ##########

# For fitting a generalised partially linear
# single index model.

# Last changed: 29 OCT 2010 by J.K. Marley and M.P. Wand.

# Issue warning about slowness:

cat("WARNING: This script takes several hours to run on\n")
cat("typical 2010 computers. On the authors' computers, in\n")
cat("2010, it takes about 12 hours to run.\n")
cat("Hit Enter to continue.\n")
ans <- readline()

# Load required package:

library(BRugs)

# Load required functions:

source("Z:/research/nonstSpr/ZOSull.r")
source("Z:/research/nonstSpr/BRugsMCMC.r")
source("Z:/research/nonstSpr/summMCMC.r")
source("Z:/research/nonstSpr/wait.r")

# Set MCMC parameters:

nBurnin <- 10000      # Length of burn-in.
nIter <- 10000        # Size of the kept sample.
nThin <- 10           # Thinning factor. 

# Set the number of knots for the smooth 
# function component:

numKnots <- 12

# Generate data corresponding to Carroll, Fan, Gijbels and Wand,
# Journal of the American Statistical Association, (1997).

framGPLSIM <- read.table("framingham.txt",header=TRUE)

y <- framGPLSIM$y
x1 <- framGPLSIM$x1 
x2 <- framGPLSIM$x2
x3 <- framGPLSIM$x3 
z <- framGPLSIM$z 
n <- length(y)

# Specify model in BUGS:

gplsimModel <- function()
{
   for(i in 1:n)
   {
      s[i] <- alpha1*x1[i] + alpha2*x2[i] + alpha3*x3[i]
      f[i] <- beta0 + betas*s[i] + inprod(u[],Z[i,])
      logit(mu[i]) <- f[i] + betaz*z[i]
      y[i] ~ dbern(mu[i])
   }
   alpha1 <- (-sin(phi)*cos(theta)) 
   alpha2 <- (-sin(phi)*sin(theta))
   alpha3 <- (-cos(phi))
   for(k in 1:numKnots)
   {
      knot[k] <- ((numKnots+1-k)*ranked(s[],1) + k*ranked(s[],n))/(numKnots+1)
      u[k] ~ dnorm(0,tauU)
      for (i in 1:n)
      {
         Z[i,k] <- (s[i]-knot[k])*step(s[i]-knot[k])
      }
   }
   phiMin <- 0 ; thetaMin <- 0
   phiMax <- pi ; thetaMax <- 2*pi
   phi ~ dunif(phiMin,phiMax) ; theta ~ dunif(thetaMin,thetaMax) 
   beta0 ~ dnorm(0,1.0E-8) ; betas ~ dnorm(0,1.0E-8)
   betaz ~ dnorm(0,1.0E-8)
   numerU ~ dnorm(0,1) ; denomU ~ dnorm(0,0.0016)
   tauU <- pow(numerU/denomU,2)
}

modelFileName <- file.path(getwd(),"gplsimModel.txt")
writeModel(gplsimModel,modelFileName)

# Set up input data:

allData <- list(n=n,y=y,x1=x1,x2=x2,x3=x3,z=z,numKnots=numKnots,pi=pi)

# Initialise parameters:

parInits <- list(list(beta0=0,betas=0,betaz=0,u=rep(0,numKnots),
                      denomU=1,numerU=1,phi=pi/2,theta=pi))

# Obtain MCMC samples for each parameter.

BRugsMCMC(data=allData,inits=parInits,
          parametersToSave=c("beta0","betas","betaz","u","phi","theta",
                             "alpha1","alpha2","alpha3","tauU","knot"),
          nBurnin=nBurnin,nIter=nIter,nThin=nThin,
          modelFile="gplsimModel.txt")

# Save MCMC samples:

beta0MCMC <- samplesSample("beta0")
betasMCMC <- samplesSample("betas")
betazMCMC <- samplesSample("betaz")
uMCMC <- NULL
for (k in 1:numKnots)
   uMCMC <- cbind(uMCMC,samplesSample(paste("u[",k,"]")))

phiMCMC <- samplesSample("phi")
thetaMCMC <- samplesSample("theta")
 
alpha1MCMC <- samplesSample("alpha1")
alpha2MCMC <- samplesSample("alpha2")
alpha3MCMC <- samplesSample("alpha3")

tauUMCMC <- samplesSample("tauU")
sigUMCMC <- 1/sqrt(tauUMCMC)

knotMCMC <- NULL
for (k in 1:numKnots)
   knotMCMC <- cbind(knotMCMC,samplesSample(paste("knot[",k,"]")))

# Do some graphical summaries and checks on the MCMC output:

parms <- list(cbind(alpha1MCMC,alpha2MCMC,alpha3MCMC,
                      betazMCMC,sigUMCMC,phiMCMC,thetaMCMC))
parNamesVal <- list(c("age","of patient"),c("transformed","blood pressure"),
                      c("logarithm of","cholesterol"),c("effect of","smoking"),
                      c(expression(sigma[u])),c(expression(phi)),
                      c(expression(theta)))

summMCMC(parms,parNames=parNamesVal)

wait()

# Draw fitted function plots:

estFunCol <- "DarkGreen" ; varBandCol <- "PaleGreen"
ng <- 101
sg <- seq(0.4,1.2,length=ng)

nMCMC <- nrow(uMCMC)
fhatMCMC <- matrix(0,ng,nMCMC)
for (g in 1:nMCMC)
{
   fhatMCMC[,g] <- beta0MCMC[g] + betasMCMC[g]*sg

   for (k in 1:numKnots)
   {
      splineg <- sg - knotMCMC[g,k] 
      splineg <- splineg*(splineg>0) 
      fhatMCMC[,g] <- fhatMCMC[,g] + uMCMC[g,k]*splineg
   }
}   
quant025 <- function(x) return(quantile(x,0.025)) 
quant975 <- function(x) return(quantile(x,0.975)) 
credLower <- apply(fhatMCMC,1,quant025)
credUpper <- apply(fhatMCMC,1,quant975)
fhatg <- apply(fhatMCMC,1,mean)
plot(sg,fhatg,type="n",ylim=range(c(credLower,credUpper)),
     xlab="single index",bty="l",
     ylab="estimated function",cex.lab=1.4)
polygon(c(sg,rev(sg)),c(credLower,rev(credUpper)),
        col=varBandCol,border=FALSE)
lines(sg,fhatg,type="l",col=estFunCol,lwd=2)

########## End gplsim ##########


