########## R script: meaErrGAM.Rs ##########

# For fitting an additive model to the Milan mortality
# data with augmented measurement error model using BRugs.

# Last changed: 29 OCT 2010 by J.K. Marley & M.P. Wand.

# Issue warning about slowness:

cat("WARNING: This script takes several days to run on typical 2009 computers.\n")
cat("On the authors' computers it takes about 10 days to run.\n")
cat("Hit Enter to continue.\n")
ans <- readline()

# Load required package:

library(BRugs)

# Load required functions:

setwd("Z:/research/nonstSpr") # Change to directory containing 
                              # files on next few lines.
source("ZOSull.r")     # For (O'Sullivan) spline basis functions.
source("BRugsMCMC.r")
source("summMCMC.r")
source("wait.r")

# Set MCMC parameters:

nBurnin <- 5000       # Length of burn-in.
nIter <- 5000         # Size of the kept sample.
nThin <- 5            # Thinning factor. 

# Read in and extract data:

milan <- read.table("milanMort.txt",header=TRUE)

y <- milan$tot.mort
x1Orig <- milan$day.num
x2Orig <- milan$mean.temp
x3Orig <- milan$rel.humid
w4Orig <- log(milan$TSP) 

# Number of observations:

n <- length(y)

# Standardised data for MCMC:

mean.x1 <- mean(x1Orig)  ; sd.x1 <- sd(x1Orig)
mean.x2 <- mean(x2Orig)  ; sd.x2 <- sd(x2Orig)
mean.x3 <- mean(x3Orig)  ; sd.x3 <- sd(x3Orig)
mean.w4 <- mean(w4Orig)  ; sd.w4 <- sd(w4Orig)

x1 <- (x1Orig - mean.x1)/sd.x1
x2 <- (x2Orig - mean.x2)/sd.x2
x3 <- (x3Orig - mean.x3)/sd.x3
w4 <- (w4Orig - mean.w4)/sd.w4

# Set up matrices for additive model:

numIntKnots <- 25
intKnots1 <- quantile(unique(x1),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x1 <- c(1.01*min(x1)-0.01*max(x1),1.01*max(x1)-0.01*min(x1))
Z1 <- ZOSull(x1,range.x1,intKnots1)
numKnots1 <- ncol(Z1)

numIntKnots <- 15
intKnots2 <- quantile(unique(x2),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x2 <- c(1.01*min(x2)-0.01*max(x2),1.01*max(x2)-0.01*min(x2))
Z2 <- ZOSull(x2,range.x2,intKnots2)
numKnots2 <- ncol(Z2)

intKnots3 <- quantile(unique(x3),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x3 <- c(1.01*min(x3)-0.01*max(x3),1.01*max(x3)-0.01*min(x3))
Z3 <- ZOSull(x3,range.x3,intKnots3)
numKnots3 <- ncol(Z3)

# Set contamination variance (Note: tauZ = 1/sigZ^2):

tauZ <- 10

# Specify model in BUGS:

meaErrGAMModel <- function()
{
   for (i in 1:n)
   {
      x4[i] ~ dnorm(muX4,tauX4)
      w4[i] ~ dnorm(x4[i],tauZ)
      log(mu[i]) <- (beta0 + beta1*x1[i] + beta2*x2[i] + beta3*x3[i] + beta4*x4[i]
                           + inprod(u1[],Z1[i,]) + inprod(u2[],Z2[i,])
                           + inprod(u3[],Z3[i,]))
      y[i] ~ dpois(mu[i])
   }


   for (k in 1:numKnots1)
   {
      u1[k] ~ dnorm(0,tauU1)
   }

   for (k in 1:numKnots2)
   {
      u2[k] ~ dnorm(0,tauU2)
   }

   for (k in 1:numKnots3)
   {
      u3[k] ~ dnorm(0,tauU3)
   }

   beta0 ~ dnorm(0,1.0E-8) ; beta1 ~ dnorm(0,1.0E-8)
   beta2 ~ dnorm(0,1.0E-8) ; beta3 ~ dnorm(0,1.0E-8)
   beta4 ~ dnorm(0,1.0E-8)
   muX4 ~ dnorm(0,1.0E-8)
   numerX4 ~ dnorm(0,1) ; denomX4 ~ dnorm(0,0.0016)
   tauX4 <- pow(numerX4/denomX4,2)
   numerU1 ~ dnorm(0,1) ; denomU1 ~ dnorm(0,0.0016)
   tauU1 <- pow(numerU1/denomU1,2)
   numerU2 ~ dnorm(0,1) ; denomU2 ~ dnorm(0,0.0016)
   tauU2 <- pow(numerU2/denomU2,2)
   numerU3 ~ dnorm(0,1) ; denomU3 ~ dnorm(0,0.0016)
   tauU3 <- pow(numerU3/denomU3,2)
}

modelFileName <- file.path(getwd(),"meaErrGAMModel.txt")
writeModel(meaErrGAMModel,modelFileName)

# Set up input data:

allData <- list(n=n,numKnots1=numKnots1,numKnots2=numKnots2,
                numKnots3=numKnots3,x1=x1,x2=x2,x3=x3,w4=w4,
                y=y,Z1=Z1,Z2=Z2,Z3=Z3,tauZ=tauZ)

# Initialise parameters:

parInits <- list(list(beta0=0,beta1=0,beta2=0,beta3=0,beta4=0,u1=rep(0,numKnots1),
                      u2=rep(0,numKnots2),u3=rep(0,numKnots3),
                      numerU1=1,denomU1=1,numerU2=1,denomU2=1,numerU3=1,denomU3=1,
                      numerX4=1,denomX4=1,muX4=0,x4=w4))

# Obtain MCMC samples using BRugs:

BRugsMCMC(data=allData,inits=parInits,parametersToSave=
          c("beta0","beta1","beta2","beta3","beta4",
            "u1","u2","u3","tauU1","tauU2","tauU3","tauX4","muX4","x4"),
            nBurnin=nBurnin,nIter=nIter,nThin=nThin,
            modelFile="meaErrGAMModel.txt")

# Save BRugs output:

betaMCMC <- samplesSample("beta0")
betaMCMC <- rbind(betaMCMC,samplesSample("beta1"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta2"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta3"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta4"))

u1MCMC <- samplesSample("u1[1]")
for (k in 2:numKnots1)
   u1MCMC <- rbind(u1MCMC,samplesSample(paste("u1[",k,"]")))

u2MCMC <- samplesSample("u2[1]")
for (k in 2:numKnots2)
   u2MCMC <- rbind(u2MCMC,samplesSample(paste("u2[",k,"]")))

u3MCMC <- samplesSample("u3[1]")
for (k in 2:numKnots3)
   u3MCMC <- rbind(u3MCMC,samplesSample(paste("u3[",k,"]")))

uMCMC <- rbind(u1MCMC,u2MCMC,u3MCMC)

tauU1MCMC <- samplesSample("tauU1") ; sigU1MCMC <- 1/sqrt(tauU1MCMC)
tauU2MCMC <- samplesSample("tauU2") ; sigU2MCMC <- 1/sqrt(tauU2MCMC)
tauU3MCMC <- samplesSample("tauU3") ; sigU3MCMC <- 1/sqrt(tauU3MCMC)

x4MCMC <- samplesSample("x4[1]")
for (i in 2:n)
   x4MCMC <- rbind(x4MCMC,samplesSample(paste("x4[",i,"]")))
estX4 <- rep(NA,length=n)
for (i in 1:n)
   estX4[i] <- mean(x4MCMC[i,])
muX4MCMC <- samplesSample("muX4")
tauX4MCMC <- samplesSample("tauX4") ; sigX4MCMC <- 1/sqrt(tauX4MCMC)

# Draw fitted function plots:

estFunCol <- "DarkGreen" ; varBandCol <- "PaleGreen"
par(mfrow=c(2,2),mai=c(1.0,0.8/2,0.1,0.1))
ng <- 101
ylimVal <- c(25,55)

# Plot first function fit (with other variables
# set at their averages):

x1g <- seq(range.x1[1],range.x1[2],length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- rep(mean(estX4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g)
f1MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f1MCMC,1,quantile,0.025)
credUpper <- apply(f1MCMC,1,quantile,0.975)
f1gOrig <- apply(f1MCMC,1,mean)
x1gOrig <- sd.x1*x1g + mean.x1
plot(x1gOrig,f1gOrig,type="n",bty="l",xlab="consecutive day number",
     ylab="mortality count",ylim=ylimVal)
polygon(c(x1gOrig,rev(x1gOrig)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(x1gOrig,f1gOrig,col=estFunCol,lwd=2)
rug(jitter(x1Orig))

# Plot second function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- seq(range.x2[1],range.x2[2],length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- rep(mean(estX4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g)
f2MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f2MCMC,1,quantile,0.025)
credUpper <- apply(f2MCMC,1,quantile,0.975)
f2gOrig <- apply(f2MCMC,1,mean)
x2gOrig <- sd.x2*x2g + mean.x2
plot(x2gOrig,f2gOrig,type="n",bty="l",xlab="mean daily temperature",
     ylab="mortality count",ylim=ylimVal)
polygon(c(x2gOrig,rev(x2gOrig)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(x2gOrig,f2gOrig,col=estFunCol,lwd=2)
rug(jitter(x2Orig))

# Plot third function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- seq(range.x3[1],range.x3[2],length=ng)
x4g <- rep(mean(estX4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g)
f3MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f3MCMC,1,quantile,0.025)
credUpper <- apply(f3MCMC,1,quantile,0.975)
f3gOrig <- apply(f3MCMC,1,mean)
x3gOrig <- sd.x3*x3g + mean.x3
plot(x3gOrig,f3gOrig,type="n",bty="l",xlab="mean daily relative humidity",
     ylab="mortality count",ylim=ylimVal)
polygon(c(x3gOrig,rev(x3gOrig)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(x3gOrig,f3gOrig,col=estFunCol,lwd=2)
rug(jitter(x3Orig))

# Plot fourth function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- seq(range(estX4)[1],range(estX4)[2],length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g)
f4MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f4MCMC,1,quantile,0.025)
credUpper <- apply(f4MCMC,1,quantile,0.975)
f4gOrig <- apply(f4MCMC,1,mean)
x4gOrig <- sd.w4*x4g + mean.w4
plot(x4gOrig,f4gOrig,type="n",bty="l",xlab="Estimated log(true Total Suspended Particles)",
     ylab="mortality count",ylim=ylimVal)
polygon(c(x4gOrig,rev(x4gOrig)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(x4gOrig,f4gOrig,col=estFunCol,lwd=2)
rug(jitter(x4gOrig))

wait()

# Do some graphical checks on the MCMC output:

parms <- list(cbind(sigU1MCMC,sigU2MCMC,sigU3MCMC,muX4MCMC,sigX4MCMC))
par.names.Val <- list(c(expression(sigma[u]),"for consecutive","day number"),
                      c(expression(sigma[u]),"for mean daily","temperature"),
                      c(expression(sigma[u]),"for mean rel.","humidity"),
                      c(expression(mu),"for true TSP"),
		      c(expression(sigma),"for true TSP"))

summMCMC(parms,par.names=par.names.Val,KDEvertLine=FALSE)

wait()

x1g <- seq(range.x1[1],range.x1[2],length=ng)
indQ1 <- length(x1g[x1g<=quantile(x1,0.25)])
f1Q1MCMC <- f1MCMC[indQ1,]
indQ2 <- length(x1g[x1g<=quantile(x1,0.50)])
f1Q2MCMC <- f1MCMC[indQ2,]
indQ3 <- length(x1g[x1g<=quantile(x1,0.75)])
f1Q3MCMC <- f1MCMC[indQ3,]

x2g <- seq(range.x2[1],range.x2[2],length=ng)
indQ1 <- length(x2g[x2g<=quantile(x2,0.25)])
f2Q1MCMC <- f2MCMC[indQ1,]
indQ2 <- length(x2g[x2g<=quantile(x2,0.50)])
f2Q2MCMC <- f2MCMC[indQ2,]
indQ3 <- length(x2g[x2g<=quantile(x2,0.75)])
f2Q3MCMC <- f2MCMC[indQ3,]

x3g <- seq(range.x3[1],range.x3[2],length=ng)
indQ1 <- length(x3g[x3g<=quantile(x3,0.25)])
f3Q1MCMC <- f3MCMC[indQ1,]
indQ2 <- length(x3g[x3g<=quantile(x3,0.50)])
f3Q2MCMC <- f3MCMC[indQ2,]
indQ3 <- length(x3g[x3g<=quantile(x3,0.75)])
f3Q3MCMC <- f3MCMC[indQ3,]

x4g <- seq(range(estX4)[1],range(estX4)[2],length=ng)
indQ1 <- length(x4g[x4g<=quantile(estX4,0.25)])
f4Q1MCMC <- f4MCMC[indQ1,]
indQ2 <- length(x4g[x4g<=quantile(estX4,0.50)])
f4Q2MCMC <- f4MCMC[indQ2,]
indQ3 <- length(x4g[x4g<=quantile(estX4,0.75)])
f4Q3MCMC <- f4MCMC[indQ3,]

parms <- list(cbind(f1Q1MCMC,f1Q2MCMC,f1Q3MCMC,f2Q1MCMC,f2Q2MCMC,f2Q3MCMC,
                    f3Q1MCMC,f3Q2MCMC,f3Q3MCMC,f4Q1MCMC,f4Q2MCMC,f4Q3MCMC))

par.names.Val <- list(c("1st quartile of","consecutive","day number"),
                      c("2nd quartile of","consecutive","day number"),
                      c("3rd quartile of","consecutive","day number"),
                      c("1st quartile of","daily mean","temperature"),
                      c("2nd quartile of","daily mean","temperature"),
                      c("3rd quartile of","daily mean","temperature"),
                      c("1st quartile of","daily relative","humidity"),
                      c("2nd quartile of","daily relative","humidity"),
                      c("3rd quartile of","daily relative","humidity"),
                      c("1st quartile of","estimated","log(true TSP)"),
                      c("2nd quartile of","estimated","log(true TSP)"),
                      c("3rd quartile of","estimated","log(true TSP)"))

summMCMC(parms,par.names=par.names.Val,KDEvertLine=FALSE)

########## End of meaErrGAM ##########
