########## R script: negBinAdd.Rs ##########

# For fitting a negative binomial additive model 
# to data on adduct counts using BRugs.

# Last changed: 29 OCT 2010 by J.K. Marley & M.P. Wand.

# Load required package:

library(BRugs)

# Load required functions:

source("Z:/research/nonstSpr/ZOSull.r")     
# For (O'Sullivan) spline basis functions.
source("Z:/research/nonstSpr/BRugsMCMC.r")
source("Z:/research/nonstSpr/summMCMC.r")
source("Z:/research/nonstSpr/BGRinterval.r")
source("Z:/research/nonstSpr/wait.r")

# Set MCMC parameters:

nBurnin <- 100000      # Length of burn-in.
nIter <- 50000         # Size of the kept sample.
nThin <- 50            # Thinning factor. 

# Read in and extract data:

adduct <- read.table("Z:/research/nonstSpr/adduct.txt",header=TRUE)

y  <- adduct$adductCount
x1Orig <- adduct$ageInit
x2Orig <- adduct$yearsSmoking
x3Orig <- adduct$yearsSinceQuit
x4Orig <- adduct$cigsPerDay

# Standardised data for MCMC:

mean.x1 <- mean(x1Orig)  ; sd.x1 <- sd(x1Orig)
mean.x2 <- mean(x2Orig)  ; sd.x2 <- sd(x2Orig)
mean.x3 <- mean(x3Orig)  ; sd.x3 <- sd(x3Orig)
mean.x4 <- mean(x4Orig)  ; sd.x4 <- sd(x4Orig)

x1 <- (x1Orig - mean.x1)/sd.x1
x2 <- (x2Orig - mean.x2)/sd.x2
x3 <- (x3Orig - mean.x3)/sd.x3
x4 <- (x4Orig - mean.x4)/sd.x4

# Set up matrices for additive model:

numIntKnots <- 10
intKnots1 <- quantile(unique(x1),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x1 <- c(1.01*min(x1)-0.01*max(x1),1.01*max(x1)-0.01*min(x1))
Z1 <- ZOSull(x1,range.x1,intKnots1)
numKnots1 <- ncol(Z1)

intKnots2 <- quantile(unique(x2),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x2 <- c(1.01*min(x2)-0.01*max(x2),1.01*max(x2)-0.01*min(x2))
Z2 <- ZOSull(x2,range.x2,intKnots2)
numKnots2 <- ncol(Z2)

intKnots3 <- quantile(unique(x3),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x3 <- c(1.01*min(x3)-0.01*max(x3),1.01*max(x3)-0.01*min(x3))
Z3 <- ZOSull(x3,range.x3,intKnots3)
numKnots3 <- ncol(Z3)

intKnots4 <- quantile(unique(x4),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x4 <- c(1.01*min(x4)-0.01*max(x4),1.01*max(x4)-0.01*min(x4))
Z4 <- ZOSull(x4,range.x4,intKnots4)
numKnots4 <- ncol(Z4)

# Specify model in BUGS:

negBinAddModel <- function()
{
   for (i in 1:n) 
   {
      log(mu[i]) <- (beta0 + beta1*x1[i]+ beta2*x2[i] + beta3*x3[i] + beta4*x4[i]
                           + inprod(u1[],Z1[i,]) + inprod(u2[],Z2[i,]) 
                           + inprod(u3[],Z3[i,]) + inprod(u4[],Z4[i,]))
      rateParm[i] <- kappa/mu[i] ; g[i] ~ dgamma(kappa,rateParm[i])   
      y[i] ~ dpois(g[i]) 
   }	

   for (k in 1:numKnots1)
   {
      u1[k] ~  dnorm(0,tauU1)
   }

   for (k in 1:numKnots2)
   {
      u2[k] ~  dnorm(0,tauU2)
   }

   for (k in 1:numKnots3)
   {
      u3[k] ~  dnorm(0,tauU3)
   }

   for (k in 1:numKnots4)
   {
      u4[k] ~  dnorm(0,tauU4)
   }

   beta0 ~ dnorm(0,1.0E-8) ; beta1 ~ dnorm(0,1.0E-8) 
   beta2 ~ dnorm(0,1.0E-8) ; beta3 ~ dnorm(0,1.0E-8)
   beta4 ~ dnorm(0,1.0E-8)
   numerU1 ~ dnorm(0,1) ; denomU1 ~ dnorm(0,0.0016)
   tauU1 <- pow(numerU1/denomU1,2)
   numerU2 ~ dnorm(0,1) ; denomU2 ~ dnorm(0,0.0016)
   tauU2 <- pow(numerU2/denomU2,2)
   numerU3 ~ dnorm(0,1) ; denomU3 ~ dnorm(0,0.0016)
   tauU3 <- pow(numerU3/denomU3,2)
   numerU4 ~ dnorm(0,1) ; denomU4 ~ dnorm(0,0.0016)
   tauU4 <- pow(numerU4/denomU4,2)
   numerKappa ~ dnorm(0,1) ; denomKappa ~ dnorm(0,0.0016)
   kappa <- pow(numerKappa/denomKappa,2)
}

modelFileName <- file.path(getwd(),"negBinAddModel.txt")
writeModel(negBinAddModel,modelFileName)

# Set up input data:

allData <- list(n=length(y),numKnots1=numKnots1,numKnots2=numKnots2,
                numKnots3=numKnots3,numKnots4=numKnots4,x1=x1,x2=x2,
                x3=x3,x4=x4,y=y,Z1=Z1,Z2=Z2,Z3=Z3,Z4=Z4)

# Initialise parameters:

parInits <- list(list(beta0=0,beta1=0,beta2=0,beta3=0,beta4=0,u1=rep(0,numKnots1),
                      u2=rep(0,numKnots2),u3=rep(0,numKnots3),u4=rep(0,numKnots4),
                      g=rep(1,length(y)),numerU1=1,denomU1=1,numerU2=1,
                      denomU2=1,numerU3=1,denomU3=1,numerU4=1,denomU4=1,
                      numerKappa=1,denomKappa=1))

# Obtain MCMC samples using BRugs:

BRugsMCMC(data=allData,inits=parInits,parametersToSave=
           c("beta0","beta1","beta2","beta3","beta4",
             "u1","u2","u3","u4","tauU1","tauU2","tauU3","tauU4","kappa"),
           nBurnin=nBurnin,nIter=nIter,nThin=nThin,
           modelFile="negBinAddModel.txt")

betaMCMC <- samplesSample("beta0")
betaMCMC <- rbind(betaMCMC,samplesSample("beta1"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta2"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta3"))
betaMCMC <- rbind(betaMCMC,samplesSample("beta4"))

u1MCMC <- samplesSample("u1[1]")
for (k in 2:numKnots1)
   u1MCMC <- rbind(u1MCMC,samplesSample(paste("u1[",k,"]")))

u2MCMC <- samplesSample("u2[1]")
for (k in 2:numKnots2)
   u2MCMC <- rbind(u2MCMC,samplesSample(paste("u2[",k,"]")))

u3MCMC <- samplesSample("u3[1]")
for (k in 2:numKnots3)
   u3MCMC <- rbind(u3MCMC,samplesSample(paste("u3[",k,"]")))

u4MCMC <- samplesSample("u4[1]")
for (k in 2:numKnots4)
   u4MCMC <- rbind(u4MCMC,samplesSample(paste("u4[",k,"]")))

uMCMC <- rbind(u1MCMC,u2MCMC,u3MCMC,u4MCMC)

tauU1MCMC <- samplesSample("tauU1") ; sigU1MCMC <- 1/sqrt(tauU1MCMC)
tauU2MCMC <- samplesSample("tauU2") ; sigU2MCMC <- 1/sqrt(tauU2MCMC)
tauU3MCMC <- samplesSample("tauU3") ; sigU3MCMC <- 1/sqrt(tauU3MCMC)
tauU4MCMC <- samplesSample("tauU4") ; sigU4MCMC <- 1/sqrt(tauU4MCMC)
kappaMCMC <- samplesSample("kappa")

# Draw fitted function plots:


estFunCol <- "DarkGreen" ; varBandCol <- "PaleGreen"
par(mfrow=c(2,2),mai=c(1.0,0.7,0.1,0.1))  # botton,left,top,right
ng <- 101
ylimVal <- c(0,300)

# Plot first function fit (with other variables
# set at their averages):

x1g <- seq(range.x1[1],(40-mean.x1)/sd.x1,length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- rep(mean(x4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Z4g <- ZOSull(x4g,range.x4,intKnots4)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g,Z4g)
f1MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f1MCMC,1,quantile,0.025)
credUpper <- apply(f1MCMC,1,quantile,0.975)
f1gOrig <- apply(f1MCMC,1,mean)
x1gOrig <- sd.x1*x1g + mean.x1
plot(x1gOrig,f1gOrig,type="n",bty="l",xlab="age of smoking initiation",
     ylab="adduct count",ylim=ylimVal)
polygon(c(x1gOrig,rev(x1gOrig)),c(credLower,rev(credUpper)),
        col=varBandCol,border=FALSE)
lines(x1gOrig,f1gOrig,col=estFunCol,lwd=2)
rug(jitter(x1Orig),quiet=TRUE)

# Plot second function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- seq(range.x2[1],range.x2[2],length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- rep(mean(x4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Z4g <- ZOSull(x4g,range.x4,intKnots4)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g,Z4g)
f2MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f2MCMC,1,quantile,0.025)
credUpper <- apply(f2MCMC,1,quantile,0.975)
f2gOrig <- apply(f2MCMC,1,mean)
x2gOrig <- sd.x2*x2g + mean.x2
plot(x2gOrig,f2gOrig,type="n",bty="l",xlab="number of years of smoking",
     ylab="adduct count",ylim=ylimVal)
polygon(c(x2gOrig,rev(x2gOrig)),c(credLower,rev(credUpper)),
        col=varBandCol,border=FALSE)
lines(x2gOrig,f2gOrig,col=estFunCol,lwd=2)
rug(jitter(x2Orig),quiet=TRUE)

# Plot third function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- seq(range.x3[1],range.x3[2],length=ng)
x4g <- rep(mean(x4),length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Z4g <- ZOSull(x4g,range.x4,intKnots4)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g,Z4g)
f3MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f3MCMC,1,quantile,0.025)
credUpper <- apply(f3MCMC,1,quantile,0.975)
f3gOrig <- apply(f3MCMC,1,mean)
x3gOrig <- sd.x3*x3g + mean.x3
plot(x3gOrig,f3gOrig,type="n",bty="l",
     xlab="number of cigarettes smoked per day",
     ylab="adduct count",ylim=ylimVal)
polygon(c(x3gOrig,rev(x3gOrig)),c(credLower,rev(credUpper)),
        col=varBandCol,border=FALSE)
lines(x3gOrig,f3gOrig,col=estFunCol,lwd=2)
rug(jitter(x3Orig),quiet=TRUE)

# Plot second function fit (with other variables
# set at their averages):

x1g <- rep(mean(x1),length=ng)
x2g <- rep(mean(x2),length=ng)
x3g <- rep(mean(x3),length=ng)
x4g <- seq(range.x4[1],range.x4[2],length=ng)
Z1g <- ZOSull(x1g,range.x1,intKnots1)
Z2g <- ZOSull(x2g,range.x2,intKnots2)
Z3g <- ZOSull(x3g,range.x3,intKnots3)
Z4g <- ZOSull(x4g,range.x4,intKnots4)
Xg <- cbind(rep(1,ng),x1g,x2g,x3g,x4g)
Zg <- cbind(Z1g,Z2g,Z3g,Z4g)
f4MCMC <- exp(Xg%*%betaMCMC + Zg%*%uMCMC)
credLower <- apply(f4MCMC,1,quantile,0.025)
credUpper <- apply(f4MCMC,1,quantile,0.975)
f4gOrig <- apply(f4MCMC,1,mean)
x4gOrig <- sd.x4*x4g + mean.x4
plot(x4gOrig,f4gOrig,type="n",bty="l",
     xlab="number of years since quitting",
     ylab="adduct count",ylim=ylimVal)
polygon(c(x4gOrig,rev(x4gOrig)),c(credLower,rev(credUpper)),
       col=varBandCol,border=FALSE)
lines(x4gOrig,f4gOrig,col=estFunCol,lwd=2)
rug(jitter(x4Orig),quiet=TRUE)
if (useWaiting) wait()

# Do some graphical checks on the MCMC output:

parms <- list(cbind(kappaMCMC,sigU1MCMC,sigU2MCMC,sigU3MCMC,sigU4MCMC))
parNamesVal <- list(c(expression(kappa)),c(expression(sigma[u1]),"age smoke init."),
                      c(expression(sigma[u2]),"years smoking"),
                      c(expression(sigma[u3]),"years since quit"),
                      c(expression(sigma[u4]),"cigs. per day"))

summMCMC(parms,parNames=parNamesVal,KDEvertLine=FALSE)
if (useWaiting) wait()

x1g <- seq(range.x1[1],(40-mean.x1)/sd.x1,length=ng)
indQ1 <- length(x1g[x1g<=quantile(x1,0.25)])
f1Q1MCMC <- f1MCMC[indQ1,]
indQ2 <- length(x1g[x1g<=quantile(x1,0.50)])
f1Q2MCMC <- f1MCMC[indQ2,]
indQ3 <- length(x1g[x1g<=quantile(x1,0.50)])
f1Q3MCMC <- f1MCMC[indQ3,]

x2g <- seq(range.x2[1],range.x2[2],length=ng)
indQ1 <- length(x2g[x2g<=quantile(x2,0.25)])
f2Q1MCMC <- f2MCMC[indQ1,]
indQ2 <- length(x2g[x2g<=quantile(x2,0.50)])
f2Q2MCMC <- f2MCMC[indQ2,]
indQ3 <- length(x2g[x2g<=quantile(x2,0.50)])
f2Q3MCMC <- f2MCMC[indQ3,]

x3g <- seq(range.x3[1],range.x3[2],length=ng)
indQ1 <- length(x3g[x3g<=quantile(x3,0.25)])
f3Q1MCMC <- f3MCMC[indQ1,]
indQ2 <- length(x3g[x3g<=quantile(x3,0.50)])
f3Q2MCMC <- f3MCMC[indQ2,]
indQ3 <- length(x3g[x3g<=quantile(x3,0.50)])
f3Q3MCMC <- f3MCMC[indQ3,]

x4g <- seq(range.x4[1],range.x4[2],length=ng)
indQ1 <- length(x4g[x4g<=quantile(x4,0.25)])
f4Q1MCMC <- f4MCMC[indQ1,]
indQ2 <- length(x4g[x4g<=quantile(x4,0.50)])
f4Q2MCMC <- f4MCMC[indQ2,]
indQ3 <- length(x4g[x4g<=quantile(x4,0.50)])
f4Q3MCMC <- f4MCMC[indQ3,]

parms <- list(cbind(f1Q1MCMC,f1Q2MCMC,f1Q3MCMC,f2Q1MCMC,f2Q2MCMC,f2Q3MCMC,
                    f3Q1MCMC,f3Q2MCMC,f3Q3MCMC,f4Q1MCMC,f4Q2MCMC,f4Q3MCMC))

parNamesVal <- list(c("1st quartile of","age smoke init."),
                      c("2nd quartile of","age smoke init."),
                      c("3rd quartile of","age smoke init."),
                      c("1st quartile of","years smoking"),
                      c("2nd quartile of","years smoking"),
                      c("3rd quartile of","years smoking"),
                      c("1st quartile of","years since quit"),
                      c("2nd quartile of","years since quit"),
                      c("3rd quartile of","years since quit"),
                      c("1st quartile of","cigs. per day"),
                      c("2nd quartile of","cigs. per day"),
                      c("3rd quartile of","cigs. per day"))

summMCMC(parms,parNames=parNamesVal,KDEvertLine=FALSE)

if (useWaiting) wait()

# Do some extra multiple chain runs and examine the
# Brooks-Gelman-Rubin diagnostic.
# Set number of multiple iterations and initial values
# for parameter of interest:

cat("Starting computations for Brooks-Gelman-Rubin convergence diagnostics...\n")

nIterMult <- 40000
numerU3Inits <- c(0.1,1,10)

# Run multiple chains:

parInitsBase <- list(beta0=0,beta1=0,beta2=0,beta3=0,beta4=0,u1=rep(0,numKnots1),
                     u2=rep(0,numKnots2),u3=rep(0,numKnots3),u4=rep(0,numKnots4),
                     g=rep(1,length(y)),numerU1=1,denomU1=1,numerU2=1,denomU2=1,
                     denomU3=1,numerU4=1,denomU4=1,numerKappa=1,denomKappa=1)

parInitsMult <- list(list=c(parInitsBase,list(numerU3=numerU3Inits[1])),
                     list=c(parInitsBase,list(numerU3=numerU3Inits[2])),
                     list=c(parInitsBase,list(numerU3=numerU3Inits[3])))

BRugsFit(data=allData,inits=parInitsMult,
         parametersToSave=c("kappa","tauU3"),
         nBurnin=0,nIter=nIterMult,nThin=1,
         modelFile="negBinAddModel.txt",numChains=3)

tauU3Mult <- samplesSample("tauU3")

# Combine into a list, with one component for each chain.
# Work on the log scale for plotting purposes:

chainInds <- list(1:(nIterMult-1),nIterMult:(2*nIterMult-2),
                 (2*nIterMult-1):(3*nIterMult-3))
MCMClist <- list(-0.5*log(tauU3Mult[chainInds[[1]]]),
                 -0.5*log(tauU3Mult[chainInds[[2]]]),
                 -0.5*log(tauU3Mult[chainInds[[3]]]))

# Plot multiple traces and Brook-Gelman-Rubin diagnostic plots:


cex.labVal <- 1.2 ;cex.mainVal <- 1.55
par(mfrow=c(3,1),mai=c(0.6,0.65,0.25,0.1))

nMCMC <- length(MCMClist[[1]])
showInds <- round(seq(1,nMCMC,length=1000))
colVec <- c("green4","darkorange","darkmagenta")
plot(0,0,type="n",xlim=c(1,nMCMC),ylim=range(unlist(MCMClist)),bty="l",
     xlab="iteration number",ylab=expression(paste("log(",sigma[u3],")")),
     main="traces of multiple chains",cex.lab=cex.labVal,cex.main=cex.mainVal)
for (j in 1:3)
   lines(showInds,MCMClist[[j]][showInds],col=colVec[j])

BGRout <- BGRinterval(MCMClist)
ylimBGR <- range(c(BGRout$numer,BGRout$denom))
plot(0,0,bty="l",type="n",xlim=range(BGRout$x),ylim=ylimBGR,
     main=
     expression(paste("traces of numerator and denominator of ",hat(R)[interval])),
     xlab="iteration number",ylab="",cex.lab=cex.labVal,cex.main=cex.mainVal)
lines(BGRout$x[showInds],BGRout$numer[showInds],col="red",lwd=2)
lines(BGRout$x[showInds],BGRout$denom[showInds],col="dodgerblue",lwd=2)
legend(0.25*nMCMC,3,legend=c("numerator","denominator"),col=c("red","dodgerblue"),
       lwd=rep(2,2))

plot(0,0,bty="l",type="n",xlim=range(BGRout$x),ylim=c(0.97,1.5),
     main=expression(paste("trace of ",hat(R)[interval])),xlab="iteration number",
     ylab=expression(hat(R)[interval]),cex.lab=cex.labVal,cex.main=cex.mainVal)
lines(BGRout$x[showInds],BGRout$numer[showInds]/BGRout$denom[showInds],
      col="chocolate",lwd=2)
abline(1,0,col="navy")

########## End of negBinAdd ##########


