######### R script: npRegMeaErr ##########

# For performing penalised spline-based scatterplot smoothing
# when the predictor is subject to measurement error. 
# A hierarchical Bayes penalised splines model is used. 

# Last changed: 29 OCT 2010 by J.K. Marley & M.P. Wand.

# Load required packages:

library(nlme)
library(BRugs)

# Load required functions:

source("Z:/research/nonstSpr/BRugsMCMC.r")
source("Z:/research/nonstSpr/summMCMC.r")
source("Z:/research/nonstSpr/wait.r")

# Set MCMC parameters:

nBurnin <- 5000      # Length of burn-in.
nIter <- 5000        # Size of the kept sample.
nThin <- 5           # Thinning factor. 

# Simulate some data

set.seed(3)
n <- 400
sigZ <- 0.1 ; tauZ <- 1/sigZ^2
sigEpsTrue <- 0.35
tauEpsTrue <- 1/sigEpsTrue^2

muXTrue <- 0.5 ; tauXTrue <- 36
sigXTrue <- 1/sqrt(tauXTrue)
x <- rnorm(n,muXTrue,sigXTrue)
w <- x + sigZ*rnorm(n)

meanfun <- function(x)
   return(sin(4*pi*x))

y <- meanfun(x) + sigEpsTrue*rnorm(n)

# Set the number of knots:

numKnots <- 22

# Specify model in BUGS:

npRegMeaErrModel <- function()
{
   for (i in 1:n) 
   {
      x[i] ~ dnorm(muX,tauX)
      w[i] ~ dnorm(x[i],tauZ)
      mu[i] <- beta0 + beta1*x[i] + inprod(u[],Z[i,])
      y[i] ~ dnorm(mu[i],tauEps)	
   }	

   for (k in 1:numKnots)
   {
      knots[k] <- ((numKnots+1-k)*ranked(x[],1)+k*ranked(x[],n))/(numKnots+1)
      for (i in 1:n)
      {
         Z[i,k] <- (x[i]-knots[k])*step(x[i]-knots[k])
      }
      u[k] ~  dnorm(0,tauU)
   }

   beta0 ~ dnorm(0,1.0E-8) ; beta1 ~ dnorm(0,1.0E-8)
   muX ~ dnorm(0,1.0E-8) 
   numerX ~ dnorm(0,1) ; denomX ~ dnorm(0,0.0016)
   tauX <- pow(numerX/denomX,2)
   numerU ~ dnorm(0,1) ; denomU ~ dnorm(0,0.0016)
   tauU <- pow(numerU/denomU,2)
   numerEps ~ dnorm(0,1) ; denomEps ~ dnorm(0,0.0016)
   tauEps <- pow(numerEps/denomEps,2)
}

modelFileName <- file.path(getwd(),"npRegMeaErrModel.txt")
writeModel(npRegMeaErrModel,modelFileName)

# Set up input data:

allData <- list(n=n,numKnots=numKnots,tauZ=tauZ,w=w,y=y)
# Initialise parameters:

parInits <- list(list(beta0=0,beta1=0,u=rep(0,numKnots),muX=0,numerX=1,
                 denomX=1,numerU=1,denomU=1,numerEps=1,denomEps=1,x=w))

# Do MCMC using BRugs:

BRugsMCMC(data=allData,inits=parInits,
          parametersToSave=c("beta0","beta1","u","tauU","tauEps",
          "muX","tauX","knots"),nBurnin=nBurnin,nIter=nIter,nThin=nThin,
          modelFile="npRegMeaErrModel.txt")

# Save BRugs output:

betaMCMC <- rbind(samplesSample("beta0"),samplesSample("beta1"))

uMCMC <- samplesSample("u[1]")
for (k in 2:numKnots)
   uMCMC <- rbind(uMCMC,samplesSample(paste("u[",k,"]")))

knotsMCMC <- samplesSample("knots[1]")
for (k in 2:numKnots)
   knotsMCMC <- rbind(knotsMCMC,samplesSample(paste("knots[",k,"]")))

tauEpsMCMC <- samplesSample("tauEps")
tauUMCMC <- samplesSample("tauU")
muXMCMC <- samplesSample("muX")
tauXMCMC <- samplesSample("tauX")
sigEpsMCMC <- 1/sqrt(tauEpsMCMC)
sigUMCMC <- 1/sqrt(tauUMCMC)
sigXMCMC <- 1/sqrt(tauXMCMC)
 
X <- cbind(rep(1,n),x)
kVec <- 1:numKnots
knots <- ((numKnots+1-kVec)*min(x)+kVec*max(x))/(numKnots+1)
Z <- outer(x,knots,"-")
Z <- Z*(Z>0)
CTC <- crossprod(cbind(X,Z)) ; Dmat <- diag(c(0,0,rep(1,ncol(Z))))
lamMCMC <- (sigEpsMCMC/sigUMCMC)^2
dfMCMC <- rep(NA,length(lamMCMC))
for (i in 1:length(lamMCMC))
   dfMCMC[i] <- sum(diag(solve(CTC+lamMCMC[i]*Dmat,CTC)))

# Do some summary plots:

parms <- list(cbind(muXMCMC,sigXMCMC,sigEpsMCMC,dfMCMC))
parNamesVal <- list(c(expression(mu[x])),c(expression(sigma[x])),
                      c(expression(sigma[epsilon])),
                      c("degrees of","freedom for f"))

summMCMC(parms,parNames=parNamesVal,KDEvertLine=FALSE,
         addTruthToKDE=c(muXTrue,sigXTrue,sigEpsTrue,-99))

wait()

# Plot curve estimate:

obsCol <- "blue" ; meaErrCol <- "orange"
estFunCol <- "DarkGreen"; trueFunCol <- "red"
varBandCol <- "PaleGreen"

par(mfrow=c(1,1),xpd=TRUE,mai=c(1.02,0.82,0.82,0.42))

ng <- 101
xg <- seq(0,0.85,length=ng)
nMCMC <- length(muXMCMC)
fMCMC <- matrix(0,ng,nMCMC)
for (g in 1:nMCMC)
{
   fMCMC[,g] <- betaMCMC[1,g]*rep(1,ng) + betaMCMC[2,g]*xg 
   for (k in 1:numKnots)
      fMCMC[,g] <- fMCMC[,g] +uMCMC[k,g]*(xg-knotsMCMC[k,g])*(xg>knotsMCMC[k,g])     
}

credLower <- apply(fMCMC,1,quantile,0.025)
credUpper <- apply(fMCMC,1,quantile,0.975)
fg <- apply(fMCMC,1,mean)

plot(xg,fg,type="n",bty="l",xlab="x",ylab="y",ylim=c(min(y),2),xlim=range(w))
polygon(c(xg,rev(xg)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(xg,fg,lwd=2,col=estFunCol)
lines(xg,meanfun(xg),lwd=2,col=trueFunCol)
points(x,y,col=meaErrCol,lwd=2)
points(w,y,col=obsCol,lwd=2)
legend("topleft",c("observed data","clean data"),col=c(obsCol,meaErrCol),
       pch=rep(1,2),pt.lwd=rep(2,2))

legend("topright",c("true f","estimated f"),lwd=rep(2,2),
       col=c(trueFunCol,estFunCol))

########## End of npRegMeaErr ##########
