######### R script: npRegMis ##########

# For performing penalised spline-based nonparametric 
# regression when the predictor is subject to missingness. 

# Last changed: 29 OCT 2010 by J.K. Marley & M.P. Wand.


# Load required package:

library(BRugs)

# Load required functions:

source("Z:/research/nonstSpr/BRugsMCMC.r")
source("Z:/research/nonstSpr/summMCMC.r")
source("Z:/research/nonstSpr/wait.r")

# Set MCMC parameters:

nBurnin <- 5000        # Length of burn-in.
nIter <- 5000          # Size of the kept sample.
nThin <- 5             # Thinning factor. 

# Set true values of parameters and mean function:

sigEpsTrue <- 0.35
fTrue <- function(x) return(sin(4*pi*x))
muXTrue <- 0.5 ; sigXTrue <- 1/6

# Simulate data using these values; with
# 20% of x data missing completely at random:

set.seed(334788)
n <- 300
misPropn <- 0.2
sigEpsTrue <- 0.35
x <- rnorm(n,muXTrue,sigXTrue)
y <- fTrue(x) + sigEpsTrue*rnorm(n)
nMis <- round(misPropn*n) ; nObs <- n - nMis
indsMis <- sample(1:n,nMis,replace=FALSE)
indsObs <- setdiff(1:n,indsMis)
xMisTrue <- x[indsMis] ; xObs <- x[indsObs]
yxMis <- y[indsMis] ; yxObs <- y[indsObs]

numKnots <- 25
knots <- seq(min(xObs),max(xObs),length=(numKnots+2))[-c(1,numKnots+2)]
  
# Obtain the Z matrix for the observed data:

ZxObs <- outer(xObs,knots,"-")
ZxObs <- ZxObs*(ZxObs>0)

# Specify model in BUGS:

npRegMisModel <- function()
{
   for (i in 1:nObs) 
   {
      muObs[i] <- beta0 + beta1*xObs[i] + inprod(u[],ZxObs[i,])
      yxObs[i] ~ dnorm(muObs[i],tauEps)	
      xObs[i] ~ dnorm(muX,tauX)
   }	
   for (i in 1:nMis) 
   {
      muMis[i] <- beta0 + beta1*xMis[i] + inprod(u[],ZxMis[i,])
      yxMis[i] ~ dnorm(muMis[i],tauEps)	
      xMis[i] ~ dnorm(muX,tauX)
   }	
   for (k in 1:numKnots)
   {
      for (i in 1:nMis)
      {
         ZxMis[i,k] <- (xMis[i]-knots[k])*step(xMis[i]-knots[k])
      }
      u[k] ~  dnorm(0,tauU)
   }
   beta0 ~ dnorm(0,1.0E-8) ; beta1 ~ dnorm(0,1.0E-8)
   muX ~ dnorm(0,1.0E-8)    
   numerX ~ dnorm(0,1) ; denomX ~ dnorm(0,0.0016)
   tauX <- pow(numerX/denomX,2)
   numerU ~ dnorm(0,1) ; denomU ~ dnorm(0,0.0016)
   tauU <- pow(numerU/denomU,2)
   numerEps ~ dnorm(0,1) ; denomEps ~ dnorm(0,0.0016)
   tauEps <- pow(numerEps/denomEps,2)
}

modelFileName <- file.path(getwd(),"npRegMisModel.txt")
writeModel(npRegMisModel,modelFileName)

# Set up input data:

allData <- list(nObs=nObs,nMis=nMis,numKnots=numKnots,xObs=xObs,
                yxObs=yxObs,yxMis=yxMis,ZxObs=ZxObs,knots=knots)

# Initialise parameters:

parInits <- list(list(beta0=0,beta1=0,u=rep(0,numKnots),muX=0,numerX=1,denomX=1,
                      numerU=1,denomU=1,numerEps=1,denomEps=1,xMis=rep(0,nMis)))

# Obtain MCMC samples using BRugs:

BRugsMCMC(data=allData,inits=parInits,
          parametersToSave=c("beta0","beta1","u","tauU","tauEps","xMis",
          "muX","tauX"),nBurnin=nBurnin,nIter=nIter,nThin=nThin,
          modelFile="npRegMisModel.txt")

# Save BRugs output:

betaMCMC <- rbind(samplesSample("beta0"),samplesSample("beta1"))
uMCMC <- samplesSample("u[1]")
for (k in 2:numKnots)
   uMCMC <- rbind(uMCMC,samplesSample(paste("u[",k,"]")))

xMisMCMC <- samplesSample("xMis[1]")
for (iMis in 2:nMis)
   xMisMCMC <- rbind(xMisMCMC,samplesSample(paste("xMis[",iMis,"]")))

tauEpsMCMC <- samplesSample("tauEps")
tauUMCMC <- samplesSample("tauU")
muXMCMC <- samplesSample("muX")
tauXMCMC <- samplesSample("tauX")
sigEpsMCMC <- 1/sqrt(tauEpsMCMC)
sigUMCMC <- 1/sqrt(tauUMCMC)
sigXMCMC <- 1/sqrt(tauXMCMC)

# Compute degrees of freedom MCMC values:

CTC <- crossprod(cbind(rep(1,nObs),xObs,ZxObs)) ; Dmat <- diag(c(0,0,rep(1,numKnots)))
lamMCMC <- (sigEpsMCMC/sigUMCMC)^2
dfMCMC <- rep(NA,length(lamMCMC))
for (i in 1:length(lamMCMC))
   dfMCMC[i] <- sum(diag(solve(CTC+lamMCMC[i]*Dmat,CTC)))

# Plot fit:

obsCol <- "blue" ; misCol <- "orange"
estFunCol <- "DarkGreen"; trueFunCol <- "red"
varBandCol <- "PaleGreen"
ng <- 101
xg <- seq(min(x),max(x),length=ng)
Xg <- cbind(rep(1,ng),xg)
Zg <- outer(xg,knots,"-")
Zg <- Zg*(Zg>0)
fMCMC <-   Xg%*%betaMCMC + Zg%*%uMCMC 
credLower <- apply(fMCMC,1,quantile,0.025)
credUpper <- apply(fMCMC,1,quantile,0.975)
fg <- apply(fMCMC,1,mean)


par(mfrow=c(1,1),mai=c(1.02,0.82,0.82,0.42))
plot(xg,fg,type="n",bty="l",xlab="x",ylab="y",xlim=range(x),ylim=range(y))
polygon(c(xg,rev(xg)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)

lines(xg,fg,lwd=2,col=estFunCol)
lines(xg,fTrue(xg),lwd=2,col=trueFunCol)
points(xObs,yxObs,col=obsCol,lwd=2)
points(xMisTrue,yxMis,col=misCol,lwd=2)
legend(0.75,1.5,c("fully observed","x value unobserved"),col=c(obsCol,misCol),
       pch=rep(1,2),pt.lwd=rep(2,2))

legend(0.467,-1.2,c("true f","estimated f"),col=c(trueFunCol,estFunCol),lwd=rep(2,2))
wait()

# Display summary of MCMC samples for model parameters:

indQ1 <- length(xg[xg<=quantile(xObs,0.25)])
fQ1MCMC <- fMCMC[indQ1,]
indQ2 <- length(xg[xg<=quantile(xObs,0.50)])
fQ2MCMC <- fMCMC[indQ2,]
indQ3 <- length(xg[xg<=quantile(xObs,0.75)])
fQ3MCMC <- fMCMC[indQ3,]

parms <- list(cbind(muXMCMC,sigXMCMC,sigEpsMCMC,dfMCMC,fQ1MCMC,fQ2MCMC,fQ3MCMC))
parNamesVal <- list(c(expression(mu[x])),c(expression(sigma[x])),
                      c(expression(sigma[epsilon])),
                      c("degrees of","freedom for f"),
                      c("first quartile","of x"),
                      c("second quart.","of x"),
                      c("third quartile","of x"))

summMCMC(parms,parNames=parNamesVal,numerSummCex=1,
            KDEvertLine=FALSE,addTruthToKDE=c(muXTrue,sigXTrue,sigEpsTrue,-99,
            fTrue(quantile(x,0.25)),fTrue(quantile(x,0.5)),
            fTrue(quantile(x,0.75))))
wait()

# Display summary for some of the missing x values:

sampInds <-  c(10,18,27,44,59)
parms <- list(t(xMisMCMC[sampInds,]))
parNamesVal <- list(c(expression(x[10]^{"mis"})),
                      c(expression(x[18]^{"mis"})),
                      c(expression(x[27]^{"mis"})),
                      c(expression(x[44]^{"mis"})),
                      c(expression(x[59]^{"mis"})))

summMCMC(parms,parNames=parNamesVal,KDEvertLine=FALSE,addTruthToKDE=x[sampInds])

########## End of npRegMis ##########
