######### R script: tNpReg ##########

# For performing robust penalised spline-based 
# nonparametric regression using a t-distribution
# model for the response variable.

# Last changed: 16 DEC 2009 by J.K. Marley & M.P. Wand.

# Load required package:

library(BRugs)

# Load required functions:

setwd("Z:/research/nonstSpr") # Change to directory containing 
                              # files on next few lines.
source("ZOSull.r")
source("BRugsMCMC.r")
source("summMCMC.r")
source("wait.r")

# Set MCMC parameters:

nBurnin <- 5000      # Length of burn-in.
nIter <- 5000        # Size of the kept sample.
nThin <- 5           # Thinning factor. 

# Set true values of parameters and mean function:

fTrue <- function(x) return(sin(4*pi*x))
set.seed(9119042)
n <- 300
x <- runif(n)
sigEpsTrue <- 0.35
nuTrue <- 1.5
y <- fTrue(x) + sigEpsTrue*rt(n,nuTrue)

# Set up Z matrix of spline basis functions:

numIntKnots <- 25
intKnots <- quantile(unique(x),seq(0,1,length=
                 (numIntKnots+2))[-c(1,(numIntKnots+2))])
range.x <- c(1.01*min(x)-0.01*max(x),1.01*max(x)-0.01*min(x))
Z <- ZOSull(x,range.x,intKnots)
numKnots <- ncol(Z)

# Specify model in BUGS:

tNpRegModel <- function()
{
   for (i in 1:n) 
   {
      mu[i] <- beta0 + beta1*x[i] + inprod(u[],Z[i,])
      precis[i] <- g[i]*tauEps
      y[i] ~ dnorm(mu[i],precis[i])
      g[i] ~ dgamma(halfNu,halfNu)	
   }	
   for (k in 1:numKnots)
   {
      u[k] ~  dnorm(0,tauU)
   }
   beta0 ~ dnorm(0,1.0E-8) ; beta1 ~ dnorm(0,1.0E-8)
   numerU ~ dnorm(0,1) ; denomU ~ dnorm(0,0.0016)
   tauU <- pow(numerU/denomU,2)
   numerEps ~ dnorm(0,1) ; denomEps ~ dnorm(0,0.0016)
   tauEps <- pow(numerEps/denomEps,2)
   recipNu ~ dbeta(1.75,2.5)
   halfNu <- 0.5/recipNu  ; nu <- 2*halfNu
}

modelFileName <- file.path(getwd(),"tNpRegModel.txt")
writeModel(tNpRegModel,modelFileName)

# Set up input data:

allData <- list(n=n,numKnots=numKnots,x=x,y=y,Z=Z)

# Initialise parameters:

parInits <- list(list(beta0=0,beta1=0,u=rep(0,numKnots),
                      g=rep(1,n),denomU=1,numerU=1,
                      denomEps=1,numerEps=1,recipNu=1))

# Obtain MCMC samples using BRugs:

BRugsMCMC(data=allData,inits=parInits,
          parametersToSave=c("beta0","beta1","u","tauU","tauEps","nu"),
          nBurnin=nBurnin,nIter=nIter,nThin=nThin,modelFile="tNpRegModel.txt")

# Save BRugs output:

betaMCMC <- rbind(samplesSample("beta0"),samplesSample("beta1"))
uMCMC <- samplesSample("u[1]")
for (k in 2:numKnots)
   uMCMC <- rbind(uMCMC,samplesSample(paste("u[",k,"]")))
tauEpsMCMC <- samplesSample("tauEps")
tauUMCMC <- samplesSample("tauU")
nuMCMC <- samplesSample("nu")
sigEpsMCMC <- 1/sqrt(tauEpsMCMC)
sigUMCMC <- 1/sqrt(tauUMCMC)

# Compute degrees of freedom MCMC values:

CTC <- crossprod(cbind(rep(1,n),x,Z)) ; Dmat <- diag(c(0,0,rep(1,numKnots)))
lamMCMC <- (sigEpsMCMC/sigUMCMC)^2
dfMCMC <- rep(NA,length(lamMCMC))
for (i in 1:length(lamMCMC))
   dfMCMC[i] <- sum(diag(solve(CTC+lamMCMC[i]*Dmat,CTC)))

# Display fitted curve:

obsCol <- "blue" 
estFunCol <- "DarkGreen"; trueFunCol <- "red"
varBandCol <- "PaleGreen"
ng <- 101
xg <- seq(range.x[1],range.x[2],length=ng)
Xg <- cbind(rep(1,ng),xg)
Zg <- ZOSull(xg,range.x,intKnots)
fMCMC <-   Xg%*%betaMCMC + Zg%*%uMCMC 
credLower <- apply(fMCMC,1,quantile,0.025)
credUpper <- apply(fMCMC,1,quantile,0.975)
fg <- apply(fMCMC,1,mean)

par(mfrow=c(1,2),mai=c(0.9,0.75,0.1,0.1))
plot(xg,fg,type="n",bty="l",xlab="x",ylab="y",xlim=range(x),ylim=range(y))
polygon(c(xg,rev(xg)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
points(x,y,col=obsCol,lwd=2)
lines(xg,fg,lwd=2,col=estFunCol)
lines(xg,fTrue(xg),lwd=2,col=trueFunCol)
legend(0.5,-7,c("true f","estimated f"),col=c(trueFunCol,estFunCol),lwd=rep(2,2))

# Display fitted curve without data (close-up):

plot(xg,fg,type="n",bty="l",xlab="x",ylab="y",xlim=range(x),
     ylim=range(c(credLower,credUpper)))
polygon(c(xg,rev(xg)),c(credLower,rev(credUpper)),col=varBandCol,border=FALSE)
lines(xg,fg,lwd=2,col=estFunCol)
lines(xg,fTrue(xg),lwd=2,col=trueFunCol)
wait()

# Display summary of MCMC samples for model parameters:

indQ1 <- length(xg[xg<=quantile(x,0.25)])
fQ1MCMC <- fMCMC[indQ1,]
indQ2 <- length(xg[xg<=quantile(x,0.50)])
fQ2MCMC <- fMCMC[indQ2,]
indQ3 <- length(xg[xg<=quantile(x,0.75)])
fQ3MCMC <- fMCMC[indQ3,]

parms <- list(cbind(sigEpsMCMC,nuMCMC,dfMCMC,fQ1MCMC,fQ2MCMC,fQ3MCMC))
parNamesVal <- list(c(expression(sigma[epsilon])),
                      c(expression(nu)),
                      c("degrees of","freedom for f"),
                      c("first quartile","of x"),
                      c("second quart.","of x"),
                      c("third quartile","of x"))

summMCMC(parms,parNames=parNamesVal,numerSummCex=1.1,KDEvertLine=FALSE,
         addTruthToKDE=c(sigEpsTrue,nuTrue,-99,fTrue(quantile(x,0.25)),
                         fTrue(quantile(x,0.5)),fTrue(quantile(x,0.75))))

########## End of tNpReg ##########
